package com.tianji.learning.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tianji.api.client.course.CourseClient;
import com.tianji.api.dto.course.CourseSimpleInfoDTO;
import com.tianji.common.domain.dto.PageDTO;
import com.tianji.common.domain.query.PageQuery;
import com.tianji.common.exceptions.BizIllegalException;
import com.tianji.common.utils.BeanUtils;
import com.tianji.common.utils.UserContext;
import com.tianji.learning.domain.po.LearningLesson;
import com.tianji.learning.domain.vo.LearningLessonVO;
import com.tianji.learning.mapper.LearningLessonMapper;
import com.tianji.learning.service.ILearningLessonService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * <p>
 * 学生课程表 服务实现类
 * </p>
 *
 * @author 东哥
 * @since 2024-11-06
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class LearningLessonServiceImpl extends ServiceImpl<LearningLessonMapper, LearningLesson> implements ILearningLessonService {
    private final CourseClient courseClient;

    /**
     * 1、添加到课程到课表（Day2 - 课堂）
     *
     * @param userId    用户ID
     * @param courseIds 课程ID集合
     */
    @Override
    @Transactional
    public void addLessons(Long userId, List<Long> courseIds) {
        // 1、根据课程ID集合远程调用查询课程信息
        List<CourseSimpleInfoDTO> courseInfoList = courseClient.getSimpleInfoList(courseIds);
        if(ObjectUtil.isEmpty(courseInfoList)){
            log.error("课程信息不存在，无法把课程添加到课表");
            return;
        }
        // 2、循环组装课表数据
        List<LearningLesson> lessons = new ArrayList<>(courseInfoList.size());
        for (CourseSimpleInfoDTO courseInfo : courseInfoList) {
            LearningLesson lesson = new LearningLesson();
            // 2.1、填充userId、courseId
            lesson.setUserId(userId);
            lesson.setCourseId(courseInfo.getId());

            // 2.2、填充课程过期时间
            Integer validDuration = courseInfo.getValidDuration();
            if(ObjectUtil.isEmpty(validDuration) || validDuration <= 0){
                continue;
            }
            lesson.setExpireTime(LocalDateTime.now().plusMonths(validDuration));
            lessons.add(lesson);
        }

        // 3、批量新增
        saveBatch(lessons);
    }

    /**
     * 2、分页查询我的课表（Day2 - 课堂2）
     *
     * @param pageQuery 分页查询参数对象
     * @return
     */
    @Override
    public PageDTO<LearningLessonVO> pageQueryMyLessons(PageQuery pageQuery) {
        // 1、获取当前登录人
        Long userId = UserContext.getUser();

        // 2、发起分页查询
        // select * from learning_lesson where user_id = #{} order by lastest_learn_time desc limit 0,5
        /*Page<LearningLesson> page = Page.of(pageQuery.getPageNo(), pageQuery.getPageSize());
        if(ObjectUtil.isEmpty(pageQuery.getSortBy())){
            page.addOrder(OrderItem.desc("latest_learn_time"));
        }else{
            page.addOrder(new OrderItem(pageQuery.getSortBy(),pageQuery.getIsAsc()));
        }*/
        Page<LearningLesson> page = this.lambdaQuery()
                .eq(LearningLesson::getUserId, userId)
                .page(pageQuery.toMpPage("latest_learn_time", false));

        List<LearningLesson> records = page.getRecords();
        if(ObjectUtil.isEmpty(records)){
            return PageDTO.empty(page);
        }

        // 3、发起远程调用，查询课程详细信息（课程名称、封面、总课时数）
        Map<Long, CourseSimpleInfoDTO> courseInfoMap = queryCourseInfoAnd2Map(records);

        // 4、循环分页结果，将po list转为 vo list
        List<LearningLessonVO> vos = new ArrayList<>(records.size());
        for (LearningLesson lesson : records) {
            // 4.1、利用工具将lesson属性拷贝至vo中
            LearningLessonVO vo = BeanUtils.copyBean(lesson, LearningLessonVO.class);

            // 4.2、获取课程特有信息（课程名称、封面、总课时数），填充至vo
            /*for (CourseSimpleInfoDTO course : courseInfoList) {
                if(course.getId().equals(lesson.getCourseId())){
                    vo.setCourseName(course.getName());
                    vo.setCourseCoverUrl(course.getCoverUrl());
                    vo.setSections(course.getSectionNum());
                    break;
                }
            }*/
            CourseSimpleInfoDTO courseInfo = courseInfoMap.get(lesson.getCourseId());
            vo.setCourseName(courseInfo.getName());
            vo.setCourseCoverUrl(courseInfo.getCoverUrl());
            vo.setSections(courseInfo.getSectionNum());

            // 4.3、存入vo集合
            vos.add(vo);
        }

        // 5、封装最终的PageDTO并返回
        return PageDTO.of(page,vos);
    }

    /**
     * 收集records中的courseId，进行远程调用，获取课程信息并转为Map
     * @param records 课表信息集合
     * @return 课程信息Map  courseId -> course
     */
    private Map<Long, CourseSimpleInfoDTO> queryCourseInfoAnd2Map(List<LearningLesson> records) {
        // 3.1、收集当前页面的所有课程ID集合
        Set<Long> courseIds = records.stream().map(LearningLesson::getCourseId).collect(Collectors.toSet());

        // 3.2、发起远程调用
        List<CourseSimpleInfoDTO> courseInfoList = courseClient.getSimpleInfoList(courseIds);

        // 3.3、健壮性判断
        if(ObjectUtil.isEmpty(courseInfoList)){
            throw new BizIllegalException("课程信息不存在！");
        }

        // 3.4、将list转换为map，共后续高效使用
        Map<Long, CourseSimpleInfoDTO> courseInfoMap = courseInfoList.stream()
                .collect(Collectors.toMap(CourseSimpleInfoDTO::getId, Function.identity()));
        return courseInfoMap;
    }
}
